package com.example.testktor.ViewModel

import android.util.Log
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateMapOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.testktor.BuySneakerRequest
import com.example.testktor.BuySneakersRequest
import com.example.testktor.Category
import com.example.testktor.SneakerCategoryResponse
import com.example.testktor.SneakerShop
import com.example.testktor.getSneakers
import io.ktor.client.HttpClient
import io.ktor.client.engine.android.Android
import kotlinx.coroutines.launch

class MainViewModel : ViewModel() {
    var sneakersCategories by mutableStateOf(SneakerCategoryResponse(emptyList(), emptyList()))
        private set

    var selectedCategory by mutableStateOf<Category?>(null)

    var likedSneakers = mutableStateMapOf<Int, Boolean>()
    var orderedSneakers = mutableStateMapOf<Int, Boolean>()

    var sneakerQuantities = mutableStateMapOf<Int, Int>()

    private val client = HttpClient(Android)

    init {
        viewModelScope.launch {
            sneakersCategories = getSneakers(client) ?: SneakerCategoryResponse(emptyList(), emptyList())
        }
    }

    fun toggleLike(sneakerId: Int) {
        likedSneakers[sneakerId] = !(likedSneakers[sneakerId] ?: false)
    }

    fun isLiked(sneakerId: Int): Boolean {
        return likedSneakers[sneakerId] ?: false
    }

    fun toggleOrder(sneakerId: Int) {
        orderedSneakers[sneakerId] = !(orderedSneakers[sneakerId] ?: false)
    }

    fun isOrdered(sneakerId: Int): Boolean {
        return orderedSneakers[sneakerId] ?: false
    }

    fun setSneakerQuantity(sneakerId: Int, quantity: Int) {
        sneakerQuantities[sneakerId] = quantity
    }

    fun getSneakerQuantity(sneakerId: Int): Int {
        return sneakerQuantities[sneakerId] ?: 1
    }

    fun getLikedSneakersByCategory(): List<SneakerShop> {
        return sneakersCategories.sneakers.filter {
            likedSneakers[it.id] == true && (selectedCategory == null || it.categoryId == selectedCategory?.id)
        }.map {
            SneakerShop(it, likeOrNot = true, orderOrNot = isOrdered(it.id))
        }
    }

    fun getOrderedSneakersByCategory(): List<SneakerShop> {
        return sneakersCategories.sneakers.filter {
            orderedSneakers[it.id] == true && (selectedCategory == null || it.categoryId == selectedCategory?.id)
        }.map {
            SneakerShop(it, likeOrNot = isLiked(it.id), orderOrNot = true)
        }
    }

    fun getLikedSneakersTotalPrice(): Double {
        return getOrderedSneakersByCategory().sumOf {
            (it.sneaker.cost * (sneakerQuantities[it.sneaker.id] ?: 1))
        }
    }

    fun removeSneakerFromCart(id: Int) {
        orderedSneakers[id] = false
        sneakerQuantities.remove(id)
    }

    fun generateBuySneakersRequest(): BuySneakersRequest {
        val orderedSneakersList = getOrderedSneakersByCategory()
        Log.d("generateBuySneakersRequest", "Кроссовки в заказе: ${orderedSneakersList.size}")
        val requestList = orderedSneakersList.map {
            Log.d("generateBuySneakersRequest", "id=${it.sneaker.id}, count=${getSneakerQuantity(it.sneaker.id)}")
            BuySneakerRequest(
                sneakerId = it.sneaker.id,
                count = getSneakerQuantity(it.sneaker.id)
            )
        }
        return BuySneakersRequest(sneakers = requestList)
    }

    fun clearCart() {
        orderedSneakers.clear()
        sneakerQuantities.clear()
    }

    fun refreshData() {
        viewModelScope.launch {
            sneakersCategories = getSneakers(client) ?: SneakerCategoryResponse(emptyList(), emptyList())
        }
    }
}