package com.example.testktor.method.main

import android.util.Log
import android.widget.Toast
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Delete
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults.buttonColors
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.dp
import androidx.navigation.NavController
import androidx.compose.material3.Text
import androidx.compose.runtime.derivedStateOf
import androidx.compose.runtime.getValue
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.unit.sp
import coil.compose.AsyncImage
import com.example.testktor.SneakerShop
import com.example.testktor.ViewModel.MainViewModel
import com.example.testktor.buySneakers
import io.ktor.client.HttpClient
import io.ktor.client.engine.android.Android
import kotlinx.coroutines.launch
import androidx.compose.ui.platform.LocalContext
import androidx.compose.foundation.Canvas

@Composable
fun BottomKorzinaNavigationBar(
    navController: NavController,
    sneakers: List<SneakerShop>,
    viewModel: MainViewModel
) {
    val context = LocalContext.current
    val coroutineScope = rememberCoroutineScope()
    val client = remember { HttpClient(Android) }

    val totalPrice by remember(sneakers) {
        derivedStateOf {
            sneakers.sumOf {
                it.sneaker.cost * viewModel.getSneakerQuantity(it.sneaker.id)
            }
        }
    }

    val totalPriceWithDelivery by remember(totalPrice) {
        derivedStateOf { totalPrice + 60 }
    }

    Box(
        modifier = Modifier
            .fillMaxSize()
            .background(Color.LightGray),
        contentAlignment = Alignment.BottomCenter
    ) {
        Column {
            Row(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(16.dp),
                verticalAlignment = Alignment.CenterVertically,
                horizontalArrangement = Arrangement.SpaceBetween
            ) {
                Text(text = "Сумма", fontSize = 18.sp)
                Text(text = "$totalPrice рубликов", fontSize = 18.sp)
            }

            Row(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(horizontal = 16.dp, vertical = 8.dp),
                verticalAlignment = Alignment.CenterVertically,
                horizontalArrangement = Arrangement.SpaceBetween
            ) {
                Text(text = "Доставка", fontSize = 18.sp)
                Text(text = "60 рубликов", fontSize = 18.sp)
            }

            Canvas(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(horizontal = 16.dp)
                    .height(1.dp)
            ) {
                val dashWidth = 10f
                val gapWidth = 10f
                var currentX = 0f

                while (currentX < size.width) {
                    drawLine(
                        color = Color.Gray,
                        start = Offset(currentX, 0f),
                        end = Offset(currentX + dashWidth, 0f),
                        strokeWidth = 2f
                    )
                    currentX += dashWidth + gapWidth
                }
            }

            Row(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(horizontal = 16.dp, vertical = 8.dp),
                verticalAlignment = Alignment.CenterVertically,
                horizontalArrangement = Arrangement.SpaceBetween
            ) {
                Text(text = "Итого", fontSize = 18.sp)
                Text(text = "$totalPriceWithDelivery рубликов", fontSize = 18.sp)
            }

            Button(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(horizontal = 16.dp, vertical = 12.dp)
                    .height(50.dp),
                shape = RoundedCornerShape(10.dp),
                colors = buttonColors(
                    containerColor = Color(0xFF48B2E7),
                    contentColor = Color.White
                ),
                onClick = {
                    coroutineScope.launch {
                        val order = viewModel.generateBuySneakersRequest()
                        Log.d("BottomKorzinaNavigationBar", "Order: $order")
                        val success = buySneakers(client, order)

                        if (success) {
                            viewModel.clearCart()
                            Toast.makeText(context, "Покупка успешно оформлена", Toast.LENGTH_SHORT).show()
                             navController.navigate("successOrderSneakers_screen")
                        } else {
                            Toast.makeText(context, "Ошибка при покупке", Toast.LENGTH_SHORT).show()
                        }
                    }
                }
            ) {
                Text("Оформить заказ", fontSize = 16.sp)
            }
        }
    }
}

@Composable
fun SneakerKorzinaButton(
    sneakerShop: SneakerShop,
    quantity: Int,
    onClick: () -> Unit,
    onIncrease: () -> Unit,
    onDecrease: () -> Unit,
    onRemove: () -> Unit
) {
    Row(
        verticalAlignment = Alignment.CenterVertically,
        modifier = Modifier.padding(horizontal = 8.dp, vertical = 4.dp)
    ) {
        Column(
            modifier = Modifier
                .background(Color(0xFF48B2E7), RoundedCornerShape(8.dp))
                .padding(4.dp),
            horizontalAlignment = Alignment.CenterHorizontally
        ) {
            IconButton(onClick = onIncrease) {
                Text("+", color = Color.White, fontSize = 18.sp)
            }
            Text("$quantity", color = Color.White, fontSize = 16.sp)
            IconButton(onClick = onDecrease) {
                Text("-", color = Color.White, fontSize = 18.sp)
            }
        }

        Spacer(modifier = Modifier.width(8.dp))

        Button(
            onClick = onClick,
            modifier = Modifier
                .width(140.dp)
                .height(180.dp),
            colors = buttonColors(containerColor = Color.White),
            shape = RoundedCornerShape(12.dp)
        ) {
            Column(horizontalAlignment = Alignment.CenterHorizontally) {
                AsyncImage(
                    model = sneakerShop.sneaker.photo,
                    contentDescription = "Sneaker Image",
                    modifier = Modifier
                        .width(200.dp)
                        .height(90.dp)
                )
                Text(text = sneakerShop.sneaker.name, fontSize = 20.sp, color = Color.DarkGray)
                Text(text = "₽${sneakerShop.sneaker.cost}", fontSize = 15.sp, color = Color.Black)
            }
        }

        Spacer(modifier = Modifier.width(8.dp))

        Box(
            modifier = Modifier
                .background(Color.Red, RoundedCornerShape(8.dp))
                .size(40.dp),
            contentAlignment = Alignment.Center
        ) {
            IconButton(onClick = onRemove) {
                Icon(
                    imageVector = Icons.Default.Delete,
                    contentDescription = "Удалить",
                    tint = Color.White
                )
            }
        }
    }
}

@Composable
fun SneakerKorzinaSelection(sneakers: List<SneakerShop>, viewModel: MainViewModel) {
    Column(modifier = Modifier.fillMaxWidth()) {
        sneakers.chunked(1).forEach { sneakerPair ->
            Row(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(vertical = 8.dp),
                horizontalArrangement = Arrangement.SpaceEvenly
            ) {
                sneakerPair.forEach { sneakerShop ->
                    val sneakerId = sneakerShop.sneaker.id
                    val quantity = viewModel.getSneakerQuantity(sneakerId)

                    SneakerKorzinaButton(
                        sneakerShop = sneakerShop,
                        quantity = quantity,
                        onClick = { /* переход */ },
                        onIncrease = {
                            viewModel.setSneakerQuantity(sneakerId, quantity + 1)
                        },
                        onDecrease = {
                            if (quantity > 1) {
                                viewModel.setSneakerQuantity(sneakerId, quantity - 1)
                            }
                        },
                        onRemove = { viewModel.removeSneakerFromCart(sneakerId) }
                    )
                }
            }
        }
    }
}